#############################################################################
## Various utilities for handling IPv6 addresses.
##
## --------------------------------------------------------------------------
## Copyright (C) 2008 The Regents of the University of California.
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
## $Id: ip6utils.rb,v 1.2 2008/12/04 02:55:22 youngh Exp $
#############################################################################

require 'ipaddr'

module ArkUtil

class IP6Utils

  # {addr} must be an integer IPv6 address, not a hexadecimal formatted string.
  # Returns the IP address packed into a string in network byte order.
  def self.pack_iton(addr)
    fail ArgumentError if addr < 0
    quads = []
    4.times do
      quads << (addr & 0xFFFFFFFF)
      addr >>= 32
    end
    quads.reverse.pack("NNNN")
  end


  # {addr} must be an ASCII (hexadecimal-formatted) IPv6 address.
  # Returns the destination as an ASCII address.
  def self.generate_random_destination(prefix, length)
    addr = IPAddr.new(prefix).to_i
    destination = generate_random_destination_aux addr, length
    IPAddr.new_ntoh(pack_iton(destination)).to_s
  end


  # {prefix} must be an integer IPv6 address, not a hexadecimal formatted
  # string.  All of the host bits of {prefix} must be zero.
  # Returns the destination as an integer.
  def self.generate_random_destination_aux(prefix, length)
    fail ArgumentError if prefix < 0 || length < 0 || length >= 128
    prefix | rand(2 ** (128 - length))
  end

end

end  # module ArkUtil
