#############################################################################
## Miscellaneous utility methods.
##
## --------------------------------------------------------------------------
## Copyright (C) 2008 The Regents of the University of California.
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
## $Id: numericutils.rb,v 1.4 2009/04/10 00:46:22 youngh Exp $
#############################################################################

module ArkUtil

  def self.compute_percentiles(values, percentiles, needs_sorting=true)
    return [] if values.length == 0 || percentiles.length == 0
    return Array.new(percentiles.length, values[0]) if values.length == 1

    retval = []
    values = values.sort if needs_sorting
    percentiles.each do |p|
      retval << compute_percentile(values, p, false)
    end
    retval
  end


  def self.compute_percentile(values, percentile, needs_sorting=true)
    if percentile <= 0 || percentile >= 100
      raise ArgumentError, "invalid percentile request"
    end
    return nil if values.length == 0
    return values[0] if values.length == 1

    values = values.sort if needs_sorting
    x = values.length * percentile
    n = x / 100
    if (x % 100) > 0
      return values[n]
    else
      return (values[n - 1] + values[n]) / 2.0
    end
  end


  def self.compute_stddev(values)
    return nil if values.length == 0
    return 0.0 if values.length == 1

    sum = values.inject(0) { |a, n| a += n }
    mean = sum.quo(values.length)

    sum_d_sq = 0.0
    values.each do |value|
      d = value - mean
      sum_d_sq += d * d
    end

    Math.sqrt(sum_d_sq / (values.length - 1))
  end

end  # module ArkUtil
